/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

// derived from Houndini sample: SOP_Surface

/** @file */

#ifndef __houdini_HoudiniSOP_h__
#define __houdini_HoudiniSOP_h__

#include <SOP/SOP_Node.h>

namespace fe
{

/**************************************************************************//**
    @brief Houdini SOP node

	@ingroup houdini
*//***************************************************************************/
class HoudiniSOP : public SOP_Node
{
	public:
								HoudiniSOP(OP_Network *net,const char *name,
										OP_Operator *op);
virtual							~HoudiniSOP(void);

static	OP_Node*				create(OP_Network *net,const char *name,
										OP_Operator *op)
								{	return new HoudiniSOP(net,name,op); }

static	int						discoverOperators(OP_OperatorTable *table);

								///	@internal
static	int						reloadCallback(void *pData,int index,float time,
										const PRM_Template* pTemplate);
	protected:
								//* as SOP_Node
virtual OP_ERROR				cookMySop(OP_Context &context);

	private:
		void					updateCatalog(Real time);


		OP_Operator*					m_pOpOperator;

		HoudiniContext					m_houdiniContext;
		sp<OperatorSurfaceI>			m_spOperatorSurfaceI;
		sp<SignalerI>					m_spSignalerI;
		sp<Layout>						m_spLayout;
		sp<DrawI>						m_spDrawI;
		Accessor< sp<Component> >		m_aSurfaceOutput;
		Accessor< sp<Component> >		m_aDrawI;
		std::vector< sp<Component> >	m_surfaceInputArray;
		sp<Component>					m_spSurfaceOutput;
		String							m_nameOutput;
		Record							m_signal;

		String							m_lastName;
		Real							m_lastTime;

		GU_Detail*						m_pGdp;
};

} // namespace

#endif /* __houdini_HoudiniSOP_h__ */
