/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __surface_SurfaceI_h__
#define __surface_SurfaceI_h__

namespace fe
{

/**************************************************************************//**
	@brief Spatial boundary

	@ingroup surface
*//***************************************************************************/
class FE_DL_EXPORT SurfaceI: virtual public Component
{
	public:
	/// @brief results of a surface intersection
	class FE_DL_EXPORT ImpactI: virtual public Counted
	{
		public:
	virtual	SpatialVector	origin(void) const								=0;

	virtual	SpatialVector	direction(void) const							=0;

	virtual	Real			distance(void) const							=0;

	virtual	SpatialVector	intersection(void)								=0;

	virtual	SpatialVector	normal(void)									=0;

	/// TEMP HACK for xMechanics test
	/// TODO Material independent from surface
	virtual	SpatialVector	diffuse(void) const								=0;
	virtual	void			setDiffuse(Color a_diffuse)						=0;
	};

							/** @brief Set limitation of surface improvement,
								such as subdivision

								Refinement may be fixed or adaptive
								and is not neccessarily uniformly applied.

								Some surfaces may ignore this setting.*/
virtual	void				setRefinement(U32 a_refinement)					=0;

							/// @brief Return the bounding center
virtual SpatialVector		center(void)									=0;
							/// @brief Return the bounding radius
virtual Real				radius(void)									=0;
							/// @brief Return the diffuse color
virtual Color				diffuse(void) const								=0;

							/** @brief Sample local transform on surface

								The notion of u and v may have limited sense
								on some surface types.  The full domain of
								u and v may not reach the full range of
								some surfaces.

								An implemention is permitted clamp u or v to
								the range 0 to 1, but may choose to extrapolate
								in one or both axes.  */
virtual SpatialTransform	sample(const SpatialTransform& a_transform,
									Vector2 a_uv) const						=0;
virtual SpatialTransform	sample(Vector2 a_uv) const						=0;

							/** @brief Sample location on surface

								An implemention may just use the translation
								from the result of sample(), but is encouraged
								to provide a more efficient limited evaluation,
								if reasonable. */
virtual SpatialVector		samplePoint(const SpatialTransform& a_transform,
									Vector2 a_uv) const						=0;
virtual SpatialVector		samplePoint(Vector2 a_uv) const					=0;

							/** @brief Sample direction from surface

								An implemention may just use the up vector
								from the result of sample(), but is encouraged
								to provide a more efficient limited evaluation,
								if reasonable. */
virtual SpatialVector		sampleNormal(const SpatialTransform& a_transform,
									Vector2 a_uv) const						=0;
virtual SpatialVector		sampleNormal(Vector2 a_uv) const				=0;

							/// @brief Closest point on the surface
virtual	sp<ImpactI>			nearestPoint(const SpatialTransform& a_transform,
									const SpatialVector& a_point)			=0;
virtual	sp<ImpactI>			nearestPoint(const SpatialVector& a_point)		=0;
virtual	sp<ImpactI>			nearestPoint(const SpatialTransform& a_transform,
									const SpatialVector& a_point)const		=0;
virtual	sp<ImpactI>			nearestPoint(const SpatialVector& a_point)const	=0;

							/** @brief Trace distance to surface in direction
										from originating point

								Impact support is not required.
								Surfaces can return a NULL ImpactI.

								const versions do not permit caching. */
virtual	sp<ImpactI>			rayImpact(const SpatialTransform& a_transform,
									const SpatialVector& a_origin,
									const SpatialVector& a_direction)		=0;
virtual	sp<ImpactI>			rayImpact(const SpatialVector& a_origin,
									const SpatialVector& a_direction)		=0;
virtual	sp<ImpactI>			rayImpact(const SpatialTransform& a_transform,
									const SpatialVector& a_origin,
									const SpatialVector& a_direction) const	=0;
virtual	sp<ImpactI>			rayImpact(const SpatialVector& a_origin,
									const SpatialVector& a_direction) const	=0;
};

} // namespace

#endif /* __surface_SurfaceI_h__ */
