/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "grass/grass.pmh"

using namespace fe;

void TwistBindOp::initialize(void)
{
	catalog<String>("Attach Attribute")="twistAttach";
	catalog<String>("Attach Attribute","hint")=
			"Name of float attribute added to output surface"
			" describing the connection of each point along"
			" its associated curve.";

	catalog< sp<Component> >("Reference Mesh");
	catalog< sp<Component> >("Reference Curves");
}

void TwistBindOp::handle(Record& a_rSignal)
{
//	feLog("TwistBindOp::handle\n");

	String attachName=catalog<String>("Attach Attribute");

	//* access output surface (copy of input surface)
	sp<SurfaceAccessibleI> spSurfaceAccessibleI=surfaceOutput(a_rSignal);
	if(spSurfaceAccessibleI.isNull())
	{
		feLog("TwistBindOp::handle"
				" output surface not spSurfaceAccessibleI\n");
		catalog<String>("error")="inaccessible output";
		return;
	}

	//* access reference curves
	sp<SurfaceAccessibleI> spReference=
			catalog< sp<Component> >("Reference Curves");
	if(spReference.isNull())
	{
		feLog("TwistBindOp::handle"
				" reference curves not spSurfaceAccessibleI\n");
		catalog<String>("error")="inaccessible reference curves";
		return;
	}

	sp<SurfaceAccessorI> spAccessor=spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_primitive,SurfaceAccessibleI::e_vertices);
	FEASSERT(spAccessor.isValid());

	sp<SurfaceAccessorI> spAccessorAttach=spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_point,attachName);
	FEASSERT(spAccessorAttach.isValid());

	sp<SurfaceAccessorI> spAccessorRef=spReference->accessor(
			SurfaceAccessibleI::e_primitive,SurfaceAccessibleI::e_vertices);
	FEASSERT(spAccessorRef.isValid());

	const U32 count=spAccessor->count();
	const U32 countRef=spAccessorRef->count();
	const U32 countMin=count<countRef? count: countRef;
	for(U32 index=0;index<countMin;index++)
	{
		const U32 subCount=spAccessor->subCount(index);
		for(U32 subIndex=0;subIndex<subCount;subIndex++)
		{
			const U32 vertexIndex=spAccessor->integer(index,subIndex);
			const SpatialVector point=spAccessor->spatialVector(index,subIndex);

			Real attach=nearestPointOnCurve(point,spAccessorRef,index);

			spAccessorAttach->set(vertexIndex,attach);
		}
	}

//	feLog("TwistBindOp::handle done\n");
}

Real TwistBindOp::nearestPointOnCurve(const SpatialVector a_point,
	const sp<SurfaceAccessorI> a_spCurves,const I32 a_index)
{
	I32 best=0;
	Real distPre2= -1.0;
	Real distBest2= -1.0;
	Real distPost2= -1.0;
	Real distLast2= -1.0;

	const U32 subCount=a_spCurves->subCount(a_index);
	for(U32 subIndex=0;subIndex<subCount;subIndex++)
	{
		const SpatialVector cv=a_spCurves->spatialVector(a_index,subIndex);
		const SpatialVector diff=cv-a_point;
		const Real dist2=magnitudeSquared(diff);
		if(!subIndex || dist2<distBest2)
		{
			best=subIndex;
			distPre2=distLast2;
			distBest2=dist2;
			distPost2= -1.0;
		}
		else if(subIndex==best+1)
		{
			distPost2=dist2;
		}

		distLast2=dist2;
	}

	Real nearest=best;
	if(best==0 || (distPost2>=0.0 && distPost2<distPre2))
	{
		if(distPost2>=0.0)
		{
			const Real distBest=sqrt(distBest2);
			const Real distPost=sqrt(distPost2);
			nearest+=distBest/(distBest+distPost);
		}
	}
	else if(distPre2>=0.0)
	{
		const Real distBest=sqrt(distBest2);
		const Real distPre=sqrt(distPre2);
		nearest-=distBest/(distBest+distPre);
	}

	return nearest;
}
