/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __vegetation_PlantSketch_h__
#define __vegetation_PlantSketch_h__

namespace fe
{

/**************************************************************************//**
	@brief Plant rendering cache

	@ingroup vegetation
*//***************************************************************************/
class FE_DL_EXPORT PlantSketch: virtual public SketchI,
		public Initialize<PlantSketch>
{
	public:
							PlantSketch(void)								{}
		void				initialize(void);
virtual						~PlantSketch(void);

#ifdef FE_VEG_VIEWER
virtual	void				bind(sp<ControllerI> spControllerI)
							{	m_spControllerI=spControllerI; }
#endif

virtual	void				bind(sp<Component> spComponent);
virtual	void				draw(sp<DrawI> spDrawI);

virtual	SpatialTransform	transform(void) const		{ return m_transform; }
virtual	void				setTransform(SpatialTransform a_transform)
							{	m_transform=a_transform; }

virtual	BWORD				paused(void) const			{ return m_paused; }
virtual	void				setPaused(BWORD paused)		{ m_paused=paused; }

	private:
		SpatialTransform		m_transform;

#ifdef FE_VEG_VIEWER
		sp<ControllerI>			m_spControllerI;
#endif

		sp<PlantModelI>			m_spPlantModelI;
		sp<DrawMode>			m_spCylinderDrawMode;
		sp<DrawMode>			m_spLineDrawMode;
		sp<DrawMode>			m_spLeafDrawMode;
		SpatialVector*			m_pOldLines;
		SpatialVector*			m_pShadowLines;
		SpatialVector*			m_pLines;
		SpatialVector*			m_pLineNormals;
		BWORD					m_paused;

		sp<Profiler>			m_spProfiler;
		sp<Profiler::Profile>	m_spProfileStartup;
		sp<Profiler::Profile>	m_spProfileCamera;
		sp<Profiler::Profile>	m_spProfileIterate;
		sp<Profiler::Profile>	m_spProfileSpan;
		sp<Profiler::Profile>	m_spProfileLineNormals;
		sp<Profiler::Profile>	m_spProfileDetails;
		sp<Profiler::Profile>	m_spProfilePoints;
		sp<Profiler::Profile>	m_spProfileCylinders;
		sp<Profiler::Profile>	m_spProfileDrawCylinders;
		sp<Profiler::Profile>	m_spProfileShadows;
		sp<Profiler::Profile>	m_spProfileText;
		sp<Profiler::Profile>	m_spProfileLines;
		sp<Profiler::Profile>	m_spProfileLeaves;
};

} /* namespace */

#endif /* __vegetation_PlantSketch_h__ */

